/*********************************************************************
* Copyright (c) 2012, Impulse Accelerated Technologies, Inc.
* All Rights Reserved.
*
* Desktop software simulation and target runtime routines
*
*********************************************************************/
#include <stdio.h>
#include "co.h"
#include "ComplexFilter.h"
#include "Filter.h"

#if defined(IMPULSE_C_TARGET)
#include <sys/time.h>
#else
#endif

#if defined(IMPULSE_C_TARGET)
struct timeval tv;
// 	struct timezone tz;
double elapsedtime_hw, elapsedtime_sw;
#endif

int k;
// Mass Storage
int32 input_data[NUM_SAMPS_IN_SLOT];
int32 if_filter_coef[IF_FILT_LEN];
int32 if_filter_output[NUM_SAMPS_IN_SLOT];

// Software ComplexFIR running in Nios II processor...
void software_fir(void)
{
	int32 coef_mem[IF_FILT_LEN];
	int32 filt_hist[IF_FILT_LEN];
	int32 outFilter;
	int32 i, jj, ii;
	int32 write_idx;
	int32 read_idx;
	int32 t_read_idx;
	int32 t_coef, t_hist;
	int16 coef_real, coef_imag;
	int16 hist_real, hist_imag;
	int32 p1_real, p2_real;
	int32 p1_imag, p2_imag;
	int32 res_real, res_imag;
	int32 accum_real, accum_imag;

#if defined(IMPULSE_C_TARGET)
	printf ("\n===================Running the Software-Only Version================\n");
	printf ("--> Begin filtering two slots\n");
	gettimeofday( &tv, NULL );
	double starttime = tv.tv_sec+tv.tv_usec*0.000001;
#else
	printf("Begin Filtering a Slot\n");
#endif

	for (i = 0; i < IF_FILT_LEN; i++)
	{
		// Fill the Coefficient Memory
		coef_mem[i] = if_filter_coef[i];
		// 		printf("...coeffients No. %d from sw: %08X\n", i, if_filter_coef[i]);
		// Clear the filter history
		filt_hist[i] = 0;
	}

	// Run NUM_SLOTS slots
	for (ii = 0; ii < NUM_SLOTS; ii++)
	{
		// Init history pointers
		write_idx = IF_FILT_LEN - 1;
		read_idx = 0;

		// Begin to process incoming data
		for (jj = 0; jj < NUM_SAMPS_IN_SLOT; jj++)
		{
#if defined(IMPULSE_C_TARGET)
			if ((jj & 0xFFFFFF80) == jj) printf("*");
#endif
			filt_hist[write_idx] = input_data[jj];
			// 			printf("......input data from sw: %08X\n", input_data[jj]);
			if (write_idx == (IF_FILT_LEN - 1))
				write_idx = 0;
			else
				write_idx++;

			t_read_idx = read_idx++;
			if (read_idx >= IF_FILT_LEN)
				read_idx = 0;

			//
			// Create Filtered value
			//

			// Clear the accums
			accum_real = 0;
			accum_imag = 0;

			i = 0;
			do
			{
				// Get the pieces
				t_coef = coef_mem[i++];
				t_hist = filt_hist[t_read_idx++];

				if (t_read_idx >= IF_FILT_LEN)
					t_read_idx = 0;

				coef_real = (t_coef >> 16) & 0xFFFF;
				coef_imag = (t_coef & 0xFFFF);
				hist_real = (t_hist >> 16) & 0xFFFF;
				hist_imag = (t_hist & 0xFFFF);

				// Create the products
				p1_real = coef_real * hist_real;
				p1_imag = coef_real * hist_imag;
				p2_real = coef_imag * hist_imag;
				p2_imag = coef_imag * hist_real;

				// Combine products & reduce bits
				res_real = p1_real - p2_real;
				res_imag = p1_imag + p2_imag;
				res_real >>= 12;
				res_imag >>= 12;

				// Accumulate products
				accum_real += res_real;
				accum_imag += res_imag;
			} while (i < IF_FILT_LEN);
			// Return result to 16 bits (assuming > 256 length filter)
			accum_real >>= 13;
			accum_imag >>= 13;

			// Transfer result to output word
			outFilter = ((accum_real & 0xFFFF) << 16) + (accum_imag & 0xFFFF);

			// Output Filtered value
			if_filter_output[jj] = outFilter;
			// 			printf("............output value from sw: %08X\n", outFilter);
		}
	}

	// Get FinishIF Timestamp
#if defined(IMPULSE_C_TARGET)
	printf ("\n--> Done filtering two slots");
	gettimeofday( &tv, NULL );
	elapsedtime_sw = tv.tv_sec+tv.tv_usec*0.000001 - starttime;
	// Print Accumulated Stats
	printf (", execution time : %f seconds\n", elapsedtime_sw);
#else
	printf("Complete Filtering a Slot\n");
#endif	
}

// Impulse declarations
extern co_architecture co_initialize(void *);

#if defined(IMPULSE_C_TARGET)
// Target will already have a main(), this will be called from there
int impc_main()
#else
int main(void)
#endif
{
#if defined(IMPULSE_C_TARGET)
#else
	char c;
#endif
	co_architecture my_arch;

	// Initialize input data array
	InitInputData(&input_data[0], (short) NUM_SAMPS_IN_SLOT);

	// Initialize IF Filter Coefficients
	InitIFFiltCoef(&if_filter_coef[0], (short) IF_FILT_LEN);

#if defined(IMPULSE_C_TARGET)
	printf("\n\n\n\n");
	printf("    ____                      __             ______\n");
	printf("   /  _/___ ___  ____  __  __/ /_______     / ____/   C-toFPGA Tools\n");
	printf("   / // __ `__ \\/ __ \\/ / / / / ___/ _ \\   / /        for Altera FPGA\n");
	printf(" _/ // / / / / / /_/ / /_/ / (__  )  __/  / /___      Platforms\n");
	printf("/___/_/ /_/ /_/ .___/\\__,_/_/____/\\___/   \\____/\n");
	printf("             /_/\n");
	printf("--------------------------------------------------------------------\n");
	printf("Complex FIR Filter Acceleration demonstration, featuring the Altera\n");
	printf("Nios II Processor, Cyclone-III FPGA and Impulse C-to-FPGA tools.\n");
#endif

	// Running on software
	software_fir();

	// Running on hardware
#if defined(IMPULSE_C_TARGET)
	// co_initialize() equivalent is called by driver initialization for NIOS II
	co_execute(NULL);
#else
	my_arch = co_initialize(NULL);
	co_execute(my_arch);
#endif

#if defined(IMPULSE_C_TARGET)
	// Print Acceleration Numbers
	printf("\n--> Acceleration factor: %.2fX\n\n", elapsedtime_sw/elapsedtime_hw);
	printf("------> Visit www.ImpulseC.com to learn more!\n\n\n");
#else
	printf("COMPLETE APPLICATION\n");
	printf("Press Enter to continue...\n");
	c = getc(stdin);
#endif
}

void call_accelerator(co_stream output_stream, co_stream input_stream)
{
	int i, j;
	int32 inSampVal;
	int32 outSampVal;
	register int32 *t_data_in_ptr;
	register int32 *t_data_out_ptr;

	co_stream_open(output_stream, O_WRONLY, INT_TYPE(32));
	co_stream_open(input_stream, O_RDONLY, INT_TYPE(32));

	// Load Filter Coefs into Accelerator
	t_data_in_ptr = if_filter_coef;
	for (i = 0; i < IF_FILT_LEN; i++)
	{
		inSampVal = *t_data_in_ptr;
		t_data_in_ptr++;
		// Write to Coef Mem
		co_stream_write(output_stream, &inSampVal, sizeof(inSampVal));
		//printf("...coeffients No. %d from sw: %08X\n", i, inSampVal);
	}

	// Get StartIF Timestamp
#if defined(IMPULSE_C_TARGET)
	printf ("\n===================Running the Accelerated Version==================\n");
	printf ("--> Begin filtering two slots\n");
	printf ("**********************************************\n");
	gettimeofday( &tv, NULL );
	double starttime = tv.tv_sec+tv.tv_usec*0.000001;
#else
	printf("Begin Filtering a Slot\n");
#endif

	// Run NUM_SLOTS slots
	for (i = 0; i < NUM_SLOTS; i++)
	{
		// Run IF Filter on 1 slot
		t_data_in_ptr = input_data;
		t_data_out_ptr = if_filter_output;
		// Send in 1st sample & start processing
		// Grab Sample from input array
		inSampVal = *t_data_in_ptr;
		t_data_in_ptr++;
		// Write to Accelerator
		co_stream_write(output_stream, &inSampVal, sizeof(inSampVal));
		//printf("......input data from sw: %08X\n", inSampVal);
		for (j = 0; j < NUM_SAMPS_IN_SLOT - 1; j++)
		{
			// Grab Sample from input array
			inSampVal = *t_data_in_ptr;
			t_data_in_ptr++;
			// Read result
			co_stream_read(input_stream, &outSampVal, sizeof(outSampVal));
			//printf("................output value from hw: %08X\n", outSampVal);
			// Write Filter output to array
			*t_data_out_ptr = outSampVal;
			t_data_out_ptr++;
			// Write to Accelerator
			co_stream_write(output_stream, &inSampVal, sizeof(inSampVal));
			//printf("......input data from sw: %08X\n", inSampVal);
		}
		// Read  last result
		co_stream_read(input_stream, &outSampVal, sizeof(outSampVal));
 		//printf("................output value from hw: %08X\n", outSampVal);
		// Write Filter output to array
		*t_data_out_ptr = outSampVal;
	}

	// Get FinishIF Timestamp
#if defined(IMPULSE_C_TARGET)
	printf ("\n--> Done filtering two slots");
	gettimeofday( &tv, NULL );
	elapsedtime_hw = tv.tv_sec+tv.tv_usec*0.000001 - starttime;
	// Print Accumulated Stats
	printf (", execution time : %f seconds\n", elapsedtime_hw);
#else
	printf("Complete Filtering a Slot\n");
#endif	

	co_stream_close(output_stream);
	co_stream_close(input_stream);
}
